//
// FortuneWnd.cpp
//

#include "FortuneWnd.h"

const uint32 MSG_BTN_ANOTHER_ONE	= 'ANOT';

// Constructor...
CFortuneWnd::CFortuneWnd()
	:BWindow(BRect(50, 50, 400, 180), "BeFortune", B_TITLED_WINDOW,
	B_ASYNCHRONOUS_CONTROLS)
{
	// Add the ScrollView and the TextView...
	BRect rectFrame = Bounds();
	rectFrame.right -= B_V_SCROLL_BAR_WIDTH;
	rectFrame.bottom -= 50;
	BRect rectText = rectFrame;
	rectText.OffsetTo(B_ORIGIN);
	
	m_pTextView = new BTextView(rectFrame, "TextView", rectText, 
			B_FOLLOW_ALL|B_WILL_DRAW);	
	m_pTextView->MakeEditable(false);

	AddChild(m_pScrollView = new BScrollView("ScrollView", m_pTextView,
			B_FOLLOW_ALL, 0, false, true));

	// Add the buttons...
	rectFrame.right += B_V_SCROLL_BAR_WIDTH;
	AddChild(m_pBtnAnotherOne = new BButton(
			BRect(10, rectFrame.bottom + 10, 
				rectFrame.right - 100, rectFrame.bottom + 40),
			"AnotherOneBtn",
			"Another one!", new BMessage(MSG_BTN_ANOTHER_ONE),
			B_FOLLOW_LEFT|B_FOLLOW_BOTTOM));

	AddChild(m_pBtnAbout = new BButton(
			BRect(rectFrame.right - 90, rectFrame.bottom + 10, 
				rectFrame.right - 10, rectFrame.bottom + 40),
			"AboutBtn",
			"About...", new BMessage(B_ABOUT_REQUESTED),
			B_FOLLOW_RIGHT|B_FOLLOW_BOTTOM));
	m_pBtnAbout->SetTarget(be_app);

	// Set size limits of the window...
	SetSizeLimits(200, 1000000, 120, 1000000);

	// Find the fortune-files...
	BPath pathEtc;
	find_directory(B_BEOS_ETC_DIRECTORY, &pathEtc);		// find BEOS/ETC-directory

	BPath pathFortune = pathEtc;
	pathFortune.Append("fortunes/default");				// append path
	
	BPath pathFortuneDat = pathEtc;
	pathFortuneDat.Append("fortunes/default.dat");		// append path
	
	// Open the fortune-data-file...
	if((m_fileFortune.SetTo(pathFortune.Path(), B_READ_ONLY) != B_OK) ||
		(m_fileFortuneDat.SetTo(pathFortuneDat.Path(), B_READ_ONLY != B_OK)))	// not succesfull?
	{
		(new BAlert("Error", "Sorry, I cannot find the fortune files!\n"
			"These files should be in your BEOS/ETC/FORTUNE-directory.\n\n"
			"The application will now quit. :-(", "Oh no!", NULL, NULL, B_WIDTH_AS_USUAL,
			B_STOP_ALERT))->Go();

		PostMessage(B_QUIT_REQUESTED);				// exit application
	}
	else
	{
		srand(system_time());		// installize random-gernerator (quick&dirty)
		
		// Calculate number of the entries in the fortune-file...
		off_t lengthFortuneDat;
		m_fileFortuneDat.GetSize(&lengthFortuneDat);	// get length of the data-file
		m_nEntries = lengthFortuneDat / sizeof(uint32);

		ShowFortune();									// show a fortune
	}
}

// MessageReceived...
void CFortuneWnd::MessageReceived(BMessage* pMsg)
{
	switch(pMsg->what)
	{
		case MSG_BTN_ANOTHER_ONE:
			ShowFortune();
			break;
		default:
			BWindow::MessageReceived(pMsg);
	}
}

// QuitRequested...
bool CFortuneWnd::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;		// close window
}

// FrameResized...
void CFortuneWnd::FrameResized(float width, float height)
{
	// Set new textrect for the TextView...
	BRect textrect = m_pTextView->TextRect();
	
	textrect.right = width - B_V_SCROLL_BAR_WIDTH;
	m_pTextView->SetTextRect(textrect);
	m_pScrollView->Invalidate();

	// Change the width of the "Another one!"-Button...
	m_pBtnAnotherOne->ResizeTo(width - 110, 30);
	m_pBtnAnotherOne->Invalidate();
}

// ShowFortune...
void CFortuneWnd::ShowFortune()
{
	// First, we calucalte a random entry-number between 0 and m_nEntries...
	int nEntry = int(float(rand()) / RAND_MAX * m_nEntries);
	
	// Now we search the offset of this entry in the text-file...
	uint32 nOffset = 0;
	m_fileFortuneDat.ReadAt(nEntry * sizeof(uint32), &nOffset, sizeof(uint32));

	// We read the first 1000 Bytes into a buffer. This is sure enough for the short
	// fortune-texts.
	const int BUFFER_SIZE = 1000;
	char* text = new char[BUFFER_SIZE];			// allocate buffer
	
	ssize_t sizeRead = m_fileFortune.ReadAt(nOffset, text, BUFFER_SIZE);	// read
	
	// Now we search for the end of the fortune-text. All fortune texts are separated 
	// by the two characters "%\n"...
	// (Special case: End Of File !)
	
	int	 nLength = 0;
	bool bPercent = false;		// Percent '%' ?
	bool bPercentAndLf = false;	// Percent and linefeed '%\n'?
	char* strEnd = text;
	
	while((++nLength < sizeRead) && !bPercentAndLf)
	{
		bPercentAndLf = (*strEnd == '\n') && bPercent;	// Percent and linefeed?	
		bPercent = *strEnd++ == '%';					// Percent?
	}
	
	m_pTextView->SetText(text, nLength - 3);			// show text
	
	delete []text;										// free buffer
}